import bpy

from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,
                       )
from bpy.types import (Panel,
                       Operator,
                       AddonPreferences,
                       PropertyGroup,
                       )
                       
haveMadeCopies = False
copiedObjects = []
objectOffsets = []
         
# ------------------------------------------------------------------------
#    store properties in the active scene
# ------------------------------------------------------------------------

class MySettings(PropertyGroup):
        
    def cyclicCurveUpdate(self, context):
        print("Cyclic Curve changed")
        curveObject = bpy.data.objects[context.scene.CurveObject]
        curveData = curveObject.data
        curveData.splines[0].use_cyclic_u = context.scene.my_tool.CurveIsCyclic

    CurveIsCyclic = BoolProperty(
        name = "Curve is Cyclic",
        description="Make curve cyclic",
        update = cyclicCurveUpdate,
        default = True,
        )
        
    ForwardPosition = BoolProperty(
        name = "Ref is First",
        description="Direction of clones",
        default = True,
        )
        
    def spacingUpdate(self, context):
        print("Spacing changed")
        if len(copiedObjects) > 0:
            if not context.scene.my_tool.UniformSpacing:
                print("Number of copied objects ",len(copiedObjects))
                for i in range(len(copiedObjects)):
                    obj = bpy.data.objects[copiedObjects[i]]
                    constraint = obj.constraints[0]
                    constraint.offset = context.scene.my_tool.Spacing
            else:
                print("Number of copied objects ",len(copiedObjects))
                spacing = float(pathLength) / (int(numObjects))
                for i in range(len(copiedObjects)):
                    obj = bpy.data.objects[copiedObjects[i]]
                    constraint = obj.constraints[0]
                    constraint.offset = spacing
        
    Spacing = FloatProperty(
        name = "Spacing between Objects",
        description = "Spacing",
        update = spacingUpdate,
        default = 10.0,
        min = 0.01,
        max = 100.0
        )
        
    numCopy = IntProperty(
        name = "Number of Copies",
        description = "Spacing",
        default = 2,
        min = 2,
        max = 10000
        )
        
    def uniformSpacingUpdate(self, context):
        context.scene.spacingUniform = not context.scene.spacingUniform
        print("Uniform spacing changed - spacingUniform = ",bpy.types.Scene.spacingUniform)
        
    UniformSpacing = BoolProperty(
        name="Uniform Spacing",
        description="Space Evenly",
        update=uniformSpacingUpdate,
        default = True
        )

class FollowingObjects(bpy.types.Panel):
    """Creates a Panel in the scene context of the properties editor"""
    bl_label = "Following Objects"
    bl_idname = "Following Objects"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "object"

    bpy.types.Object.numDup = bpy.props.FloatProperty(name="Number of Copies")
             
    def draw(self, context):
        layout = self.layout
        scene = context.scene  
        obj = context.object
        myTool = scene.my_tool
 
        
        layout.prop_search(scene, "BaseObject", scene, "objects", icon='MESH_CUBE')
        
        row = layout.row()

        layout.prop_search(scene, "CurveObject", scene, "objects", icon='OUTLINER_OB_CURVE')
        layout.prop(myTool,"CurveIsCyclic",text="Make curve cyclic")
        
        layout.prop(myTool,"numCopy",text="Number of Copies")
        
        layout.prop(myTool,"UniformSpacing",text="Uniform Spacing")

        row = layout.row()
        layout.prop(myTool,"Spacing",text="Spacing between Objects")
        
        row.enabled = True
        layout.prop(myTool,"ForwardPosition",text="Base object is first inline")
        
        row = layout.row()      
        row.label(text="Select Follow Path target direction")  
        layout.prop(scene,"my_targetDir")
        row = layout.row()      
        row.label(text="Select Up Direction")  
        layout.prop(scene,"my_upDir")
        
        row = layout.row()
        
        row.operator("my.button", text="Create Copies")
                           

def register():
    bpy.utils.register_module(__name__)
    bpy.types.Scene.my_tool = PointerProperty(type=MySettings)
    bpy.types.Scene.BaseObject = bpy.props.StringProperty()
    bpy.types.Scene.CurveObject = bpy.props.StringProperty()
    bpy.types.Scene.my_targetDir = bpy.props.EnumProperty(
            name = "My enum",
            description = "My enum description",
            default = {'FORWARD_X'},
            items = [
                ("FORWARD_X" , "X" , ""),
                ("FORWARD_Y", "Y", ""),
                ("FORWARD_Z", "Z", ""),
                ("TRACK_NEGATIVE_X", "- X", ""),
                ("TRACK_NEGATIVE_Y", "- Y", ""),
                ("TRACK_NEGATIVE_Z", "- Z", "")
            ],
            options = {"ENUM_FLAG"}
        )
    bpy.types.Scene.my_upDir = bpy.props.EnumProperty(
            name = "My enum",
            description = "My enum description",
            default = {'ZUP'},
            items = [
                ("XUP" , "X" , ""),
                ("YUP", "Y", ""),
                ("ZUP", "Z", "")
            ],
            options = {"ENUM_FLAG"}
        )
    bpy.types.Scene.spacingUniform = BoolProperty(name='spacingFlag', default = False)
        
def unregister():
    bpy.utils.unregister_module(__name__)
    del bpy.types.Object.BaseObject
    del bpy.types.Object.CurveObject
    del bpy.types.Scene.my_tool
    del bpy.types.Scene.spacingUniform 


if __name__ == "__main__":
    register()